/**
  ******************************************************************************
	By E.M. SILICA ITALY
	Date: 19-09-2009
	Vers. 1.0.0
	
	Software used:
	STLIB 1.1.1 + STVD 4.1.3 + C Compiler COSMIC
	
	Board used: 
	SILICA-STM8 demo board ( XYZSILICASTM8LCE!A )
	
	ADC used in Interrupt mode to read Potentiometer
	TIM4 (SysTick) for drive LEDs connected on PI6 and PI7
	RS485 which sends the ADC value (0,1,2,3,4) to another board that receives 
				and displays it
	RS485 parameters
				- BaudRate = 9600 baud  
				- Word Length = 8 Bits
				- One Stop Bit
				- NO parity
	******************************************************************************
  */
	

/* Includes ------------------------------------------------------------------*/
#include "stm8s.h"
#include "main.h"

 int ContTim4=0;						// Variable for Time Base for TIM4
 u16 Conversion_Value = 0;
 char StatoLed = 0;
 char ValADtoSend_Pre = 0;
 char ValADtoSend = 0;/*							Led1	Led2	Led3	Led4	Valore
											==0x0 					0			0			0			0				0
											>0x0 <0xFF			1			0			0			0				1
											>0x100 <0x1FF		1			1			0			0				2
											>0x200 <0x2ff 	1			1			1			0				3
											else						1			1			1			1				4
										*/
 


void Dly(u16 Cont);
void Set485toTX(void);
void Set485toRX(void);
void Tx485Char(char val);


// Generic delay affects only WHILE loop and the freq. of MCU
void Dly(u16 Cont)
{
	while (Cont > 1)
		Cont--;
}


// Places RS485 TX and do a small delay
void Set485toTX(void)
{
	Set485OutPut	// USART3 in OutPut
	Dly(DLY_Switch485_RxTx_or_TxRx);
}


// Places RS485 RX and do a small delay
void Set485toRX(void)
{
	Set485InPut		// USART3 in InPut
	Dly(DLY_Switch485_RxTx_or_TxRx);
}


// Transmits the received character (char val) and also do a slight delay because
// if you use a PC as a receiver you risk losing data for the slowness of USB-RS485 converter
void Tx485Char(char val)
{
	Set485toTX();	// Set 485 to TX
	while (!UART3_GetFlagStatus(UART3_FLAG_TC));		
	UART3_SendData8(val);
	while (!UART3_GetFlagStatus(UART3_FLAG_TC));			
	Dly(DLY_DopoTxdiUnCarattere);
	Set485toRX(); // Set 485 to RX
}



void main(void)
{
	
		// Clock Esterno da quarzo 24MHz HSE diviso 4
		CLK_DeInit();
		CLK_HSECmd(ENABLE);
		CLK_SYSCLKConfig(CLK_PRESCALER_CPUDIV4);
		
		// UART3 configuration ------------------------------------------------------
		/* UART3 configured as follow:
		- BaudRate = 9600 baud  
		- Word Length = 8 Bits
		- One Stop Bit
		- NO parity
		- Receive and transmit enabled
		*/
		UART3_DeInit();
		UART3_Init((u32)9600, UART3_WORDLENGTH_8D, UART3_STOPBITS_1, UART3_PARITY_NO, UART3_MODE_TXRX_ENABLE);
		UART3_ITConfig(UART3_IT_RXNE_OR, ENABLE);
		UART3_Cmd(ENABLE);
	
		// Configure SysTick TIM4
		// Time base configuration at 1mS  
		TIM4_TimeBaseInit(TIM4_PRESCALER_16, 0x7F);
		TIM4_ITConfig(TIM4_IT_UPDATE, ENABLE);
		TIM4_Cmd(ENABLE);	// Enable TIM4 	
	
		/* Init GPIO in OutPut Mode for RS485 DIR Pin  */
		GPIO_Init(RS485_PORT, DIR485, GPIO_MODE_OUT_PP_LOW_FAST);
		// Set the RS485 in InPut mode
		Set485InPut	
	
    /*  Init GPIO for ADC2 */
    GPIO_Init(GPIOE, GPIO_PIN_6, GPIO_MODE_IN_FL_NO_IT);

    /* Init GPIO for LED  */
    GPIO_Init(LEDS_PORT, (LED1_PIN |LED2_PIN |LED3_PIN |LED4_PIN), GPIO_MODE_OUT_PP_LOW_FAST);
		GPIO_Init(GPIOI, (GPIO_PIN_6 |GPIO_PIN_7), GPIO_MODE_OUT_PP_LOW_FAST);
		// TurnOFF all LEDs
		GPIO_WriteHigh(LEDS_PORT, (LED1_PIN |LED2_PIN |LED3_PIN |LED4_PIN));
		GPIO_WriteHigh(GPIOI, (GPIO_PIN_6 |GPIO_PIN_7));
		
    enableInterrupts();
    
		// Configure ADC +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		ADC2_DeInit();
		/* Init ADC2 peripheral */
		ADC2_Init(ADC2_CONVERSIONMODE_CONTINUOUS, ADC2_CHANNEL_9, ADC2_PRESSEL_FCPU_D18, ADC2_EXTTRIG_TIM, DISABLE, ADC2_ALIGN_RIGHT, ADC2_SCHMITTTRIG_CHANNEL9, DISABLE);
		/* Enable EOC interrupt */
		ADC2_ITConfig(ENABLE);
		/*Start Conversion */
		ADC2_StartConversion();
		// END Configure ADC +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		
    while (1)
		{
		// Check if is necessary send the value of AD via RS485
		if (ValADtoSend_Pre != ValADtoSend)
			{
			ValADtoSend_Pre=ValADtoSend;
			Tx485Char(ValADtoSend);
			}
		
		// Check the character received via RS485
		if (StatoLed=='0') // All LEDs OFF
			GPIO_WriteHigh(LEDS_PORT, (LED1_PIN |LED2_PIN |LED3_PIN |LED4_PIN));
		if (StatoLed=='1')
			{
			GPIO_WriteLow(LEDS_PORT, LED1_PIN);
			GPIO_WriteHigh(LEDS_PORT, LED3_PIN|LED2_PIN|LED4_PIN);
			}
		if (StatoLed=='2')
			{
			GPIO_WriteLow(LEDS_PORT, LED2_PIN|LED1_PIN);
			GPIO_WriteHigh(LEDS_PORT, LED3_PIN|LED4_PIN);
			}
		if (StatoLed=='3')
			{
			GPIO_WriteLow(LEDS_PORT, LED3_PIN|LED2_PIN|LED1_PIN);
			GPIO_WriteHigh(LEDS_PORT, LED4_PIN);
			}
		if (StatoLed=='4')	// ALL LEDs ON
			GPIO_WriteLow(LEDS_PORT, (LED1_PIN |LED2_PIN |LED3_PIN |LED4_PIN));
		
		}
		
}

#ifdef USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval 
  * None
  */
void assert_failed(u8* file, u32 line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
		
  }
}
#endif

/**
  * @}
  */

/******************* (C) COPYRIGHT 2009 STMicroelectronics *****END OF FILE****/
